unit DS048u1;

(*N.B: According to the TMEX ver 3.10 API help file, the following "type"
    names in the TMEX documentation are used as indicated... I have
    prefixed each line with the equivalent Delphi (ver 4, anyway) type

    BUT! You have to take some of this with a pinch... or more... or salt,
    unless I'm missing something.

    In the TMExtendedStartSession documentation, it clearly says that
    if the function returns a negative 1, then it means (something).

    But we are told to assign what TMExtendedStartSession returns to
    a variable of type "long", in the example.

    However, elsewhere, we were told that "long", in their terms, is
    a 32-bit UNSIGNED integer...???

    Apply common sense, and some optimism, I guess.

    To be fair, I have skirted over the word "far" in one part of
    the documentation... but in the example of using
    TMExtendedStartSession, the "far" is absent.... to quote...

long session_handle;
short PortNum = 1, PortType = 1;

/* attempt to get a session on Port */
session_handle = TMExtendedStartSession(PortNum,PortType,NULL);

if (session_handle > 0)
{
    /* call TMEX API function with session_handle */
    ...
}
else if (session_handle == 0)
{
    /* port not available */
    ...
}
else if (session_handle == -1)    /* <<<<<<<<<<<<<
{
    /* failure, indicated type does not have a driver */
    ...

End quote. I rest my case. As I said... hope and pray, but,
they SAY.....

    What Delphi  TMEX      .... i.e...
    calls...     calls...

    word         short     16 bit unsigned integer
    longword     long      32 bit unsigned integer

    ===
    Delphi further says...

    Delphi type   ... is...
    Smallint      16 bit signed integer
    Longint       32 bit signed integer

    *)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Inifiles, iBTMEXPW;
  //N.B.: This code will only compile if at least iBTMEXPW.dcu is available.
  //(I usually just put a copy in the project's sourcecode folder. The .dcu
  // comes (free) from Maxim/ Dallas as part of the SDK/ 1-Wire drivers.

const vers='1/1/14b';
      //started 30 Dec 13, along road to DS025B
      sReadyToTry='You have POSSIBLY valid data. But, while, for example, your port number IS a number, it may not be the RIGHT number';
      sReadyToTryNOT='Enter numbers for Port and Adapter and string of 16 hex characters for chip ID.';

type
  TDS048f1 = class(TForm)
    laPortNum: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    ePortNum: TEdit;
    eAdapterType: TEdit;
    eChipID0: TEdit;
    buDoTturePt1: TButton;
    buQuit: TButton;
    laReadChipEnableInfo: TLabel;
    laRes0: TLabel;
    laRes1: TLabel;
    laRes2: TLabel;
    laRes3: TLabel;
    eChipID1: TEdit;
    Label1: TLabel;
    GroupBox1: TGroupBox;
    rbReadChip0: TRadioButton;
    rbReadChip1: TRadioButton;
    buDoTturePt2: TButton;
    laTtureResult: TLabel;
    procedure ePortNumChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure buQuitClick(Sender: TObject);
    procedure buDoTturePt1Click(Sender: TObject);
    procedure eAdapterTypeChange(Sender: TObject);
    procedure eChipID0Change(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure eChipID1Change(Sender: TObject);
    procedure buDoTturePt2Click(Sender: TObject);
  private
    { Private declarations }
    boGoodPortNumber,boGoodAdapterType,boGoodChipID0,boGoodChipID1:boolean;
    wTtureCentiK:word;
    
    //Variables specifically for TMEX
    wPortNum,wAdapterType:word;
    liSessionHandle:longint;//TMEX API notes called for 32-bit
       //UNsigned... but then gave example where "if (it)<0 then..."
       //appeared. So I am using Delphi longint, which is 32 bit SIGNED
    siResult:smallint;////TMEX API notes called for 16-bit
       //UNsigned... but then (in TMSessionEnd.. etc?) gave example
       //where "if (it)<0 then..." appeared. So I am using
       // Delphi smallint, which is 16 bit SIGNED
    siNumbersForTMRom:array [0..8] of smallint;//Be careful... Delphi also has
                                     //a shortint... but it is different.
                                     //We want, here and for siResult, and
                                     //smallint gives to us, a
                                     //16-bit signed integer type.
                                     //Used by TMRom
    bTMEXstatebuffer:array[0..15360] of byte;//Used by TMRom, TMAccess

    //end TMEX variables

    sChipID: array[0..1] of string[16];
    procedure SeeIfTimeToEnableReadChip;
    procedure TalkTo1Wire(wWhatAction:word);
       //When TalkTo1Wire executed, much will be the same every time.
       //At the heart of it, there will be a "switch" controlled
       //      by wWhatAction to determine what messages go to
       //      the chip to which we have made a connection.
       //The values in wWhatAction are arbitrary, but I will usually
       //      use the number of the command I intend to issue as the
       //       "code" for the wWhatAction "switch" value.
       //See top of TalkTo1Wire for defined wWhatAction

    procedure KludgeDelay;
    function sTtureCfrmCentiK(wTture:word):string;
    function wRawTTureToCentiK(bTtureLo,bTtureHi,bChipFam:byte):word;

  public
    { Public declarations }
  end;

var
  DS048f1: TDS048f1;

implementation

{$R *.DFM}
{$R+}

procedure TDS048f1.FormCreate(Sender: TObject);
var dfIniFile:TIniFile;//"df" in name from "datafile".
begin
wPortNum:=999;//Init
wAdapterType:=999;//Init
laRes0.caption:='';
laReadChipEnableInfo.caption:=sReadyToTryNOT;
boGoodPortNumber:=false;
buDoTturePt1.enabled:=false;
buDoTturePt2.enabled:=false;

//--------------------------------------------------------------
//Read from ini file, or, ini file absent, use hard-coded defaults here.
//Note: There are OnChange handlers on many of the edit boxes

dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+'DS048ini.txt');

with dfIniFile do begin
   ePortNum.text:=ReadString('Port','Number','2');
   eAdapterType.text:=ReadString('Port','AdapterType','2');
   eChipID0.text:=ReadString('Chip','ID-Chip A','1111222233334444');
   eChipID1.text:=ReadString('Chip','ID-Chip B','1111222233334444');
   end;//with...

dfIniFile.Free;
//End of "Read from ini..."

end;

procedure TDS048f1.SeeIfTimeToEnableReadChip;
begin
if boGoodPortNumber and
   boGoodAdapterType and
   boGoodChipID0 and
   boGoodChipID1 then begin
     laReadChipEnableInfo.caption:=sReadyToTry;
     buDoTturePt1.enabled:=true;
     buDoTturePt2.enabled:=true;
     end//no ; here
   else begin
     laReadChipEnableInfo.caption:=sReadyToTryNOT;
     buDoTturePt1.enabled:=false;
     buDoTturePt2.enabled:=false;
     end;
end;//SeeIfTimeToEnableReadChip

procedure TDS048f1.buQuitClick(Sender: TObject);
begin
close;
end;

procedure TDS048f1.ePortNumChange(Sender: TObject);
begin
boGoodPortNumber:=true;
try
  wPortNum:=StrToInt(ePortNum.text);
  except
  boGoodPortNumber:=false;
  end;//try.. except
SeeIfTimeToEnableReadChip;
laRes2.caption:=inttostr(wPortNum);
end;//ePortNumChange

procedure TDS048f1.eAdapterTypeChange(Sender: TObject);
begin
boGoodAdapterType:=true;
try
  wAdapterType:=StrToInt(eAdapterType.text);
  except
  boGoodAdapterType:=false;
  end;//try.. except
SeeIfTimeToEnableReadChip;
end;//eAdapterTypeChange


procedure TDS048f1.FormClose(Sender: TObject; var Action: TCloseAction);
var dfIniFile:TInifile;
begin
 dfIniFile:=TIniFile.Create(ExtractFilePath(Application.ExeName)+'DS048ini.txt');
 with dfIniFile do begin
   WriteString('Port','Number',ePortNum.text);
   WriteString('Port','AdapterType',eAdapterType.text);
   WriteString('Chip','ID-Chip A',eChipID0.text);
   WriteString('Chip','ID-Chip B',eChipID1.text);
   end;//with...
 dfIniFile.Free;

end;//FormClose

procedure TDS048f1.buDoTturePt1Click(Sender: TObject);
begin
TalkTo1Wire($44);//Do a "Convert Tture". Should include "and wait 500ms", in
//case on parasitic. Can come out, if speed needed, but will
//require some special stuff.
end;// buDoTturePt1Click

procedure TDS048f1.buDoTturePt2Click(Sender: TObject);
begin
TalkTo1Wire($BE);//Do a "ReadScratchpad".
//For now, puts two of the read bytes into special global variables...
//   the bytes returned should go someplace more general.
//Presumes a prior "Convert Tture"
end;//DoTturePt2Click

procedure TDS048f1.TalkTo1Wire(wWhatAction:word);
//This is a rough outline, to show the "shape" of the TalkTo1Wire routine

//When TalkTo1Wire executed, much will be the same every time.
//At the heart of it, there will be a "switch" controlled
//      by wWhatAction to determine what messages go to
//      the chip to which we have made a connection.
//The values in wWhatAction are arbitrary, but I will usually
//      use the number of the command I intend to issue as the
//       "code" for the wWhatAction "switch" value.
//wWhatAction set to 0 inside each handler, as part of "was undefined
//      wWhatAction requested" handling.
//Defined wWhatActions...
//  $44: Convert Tture. Valid for DS1820 (family $10)
//                                DS18B20 (family $28)
//       Issues a "Convert Tture" command... and(?) does a delay?
//                                       ... and(?) strong pullup?
//  $5E: Read Scratchpad. Valid for, at least, DS1820, DS18B20
//      At 1/1/14, it was putting results in too-specific global vars
//      Needs a "check checksum" part.

var bTmpL,bErrL,bErrl1:byte;
    boWhatActionHandlerFound:boolean;
    sErrMsgL:string;
    bTtureLo,bTtureHi:byte;//KLUDGE-rework in more general state
procedure DoWhatReportingOfErrorYouWantTo;//SR of ReadChip
begin
showmessage('Write code for DoWhatReportingOfErrorYouWantTo'+chr(13)+
  'bErrL='+inttostr(bErrL)+chr(13)+
  'bErrL1='+inttostr(bErrL1)+chr(13)+
  'sErrMsgL='+sErrMsgL);
laRes0.caption:='Read chip AS FAR AS IT SO FAR GOES was **NOT** successful.';
laRes1.caption:='';
end;//DoWhatReportingOfErrorYouWantTo... SR of ReadChip

procedure DoThingsIfNeeded;//SR of ReadChip
begin
laRes0.caption:='ReadChip still needs more work';
laRes1.caption:='Read chip AS FAR AS IT SO FAR GOES was successful.';
end;//DoThingsIfNeeded... SR of ReadChip

procedure FillsiNumbersForTMRom(bWhichChip:byte);//SR of ReadChip
var bCount:byte;
//NB: "Family code" goes in siNumbersForTMRom[0], and rest fill with
//  the parts of the ID code from family code end to other end.
//I.e. if the family code were 28, and the chip ID were...
//  1122334455667728, then  siNumbersForTMRom[0],[1],[2], etc would be..
//  28, 77, 66, 55, 44, 33... etc, with 11 going in siNumbersForTMRom[7]
begin
for bCount:=0 to 7 do begin
   siNumbersForTMRom[7-bCount]:=strtoint('$'+copy(sChipID[bWhichChip],(bCount*2)+1,2));
// showmessage(inttostr(strtoint('$'+copy(sChipID[bWhichChip],(bCount*2)+1,2)))); // <<for prgm testing
   end;//for
end;//FillsiNumbersForTMRom... //SR of ReadChip

begin //ReadChip
//This is a rough outline, to show the "shape" of the ReadChip routine

bErrL:=0; //To initialize thess flag variables
bErrL1:=0;//"0" in bErrL means no error seen to this point
          //If bErrL<>0, then value in bErrL1 may give further information,
          //   see sourcecode... what you are reading now!
sErrMsgL:='No Error Seen';

//===AAAAA==========================
//Call TMExtendedStartSession

//showmessage(inttostr(wPortNum)+'  '+inttostr(wAdapterType));<<for prgm testing

liSessionHandle:=TMExtendedStartSession(wPortNum,wAdapterType,nil);
//Note the "nil" as the third parameter. Right for Pascal (Delphi/ Lazarus)
//  in place of "null" in other languages

//In tests, at least with a LinkUSB adaptere from iButtonLink,
//  TMExtendedStartSession seemed remarkably forgiving! Many times it
//  "should" not have worked due to bad values in wPortNum or wAdapterType
//  it DID work. Hmmm.

if liSessionHandle=0 then bErrL:=10;//Port not available.
if liSessionHandle<0 then begin
       bErrL:=12;//Other error, see TMEX Session Error Return Codes
          bErrL1:=240;
          if liSessionHandle=-200 then begin
             bErrL1:=200;// INVALID_SESSION -  session not valid
             sErrMsgL:='INVALID_SESSION -  session not valid';
             end;
          if liSessionHandle=-201 then begin
             bErrL1:=201;// HS_NOT_FOUND -  Hardware_Specific driver not found and is required
             sErrMsgL:='HS_NOT_FOUND -  Required Hardware_Specific driver not found';
             end;
          if bErrL1=240 then begin
             bErrL1:=1;//Other error encountered (Not in Dallas docs
              //for TMEX API ver 3.10)
             sErrMsgL:='Other error, not in Dallas docs, TMEX API ver 3.10';
             end;
        end;//liSessionHandle<0
          //end of flag problems code for call of TMExtendedSession

//N.B. If (bErrL=0) or (bErrL>19) later (at TMEndSession) we will call
//  TMEndSession to close the session, even if we didn't manage to
//  do with it all that we wanted to.
//  Only for bErrL:=1 to 19 (inclusive) will we NOT call TMEndSession,
//  because those error codes are reserved to say "we didn't succeed in
//  opening a session.

//TO BE DECIDED... does it matter if we do EXTRA TMEndSessions? As in the case
//  where a session ceased to be valid. I would be inclined to think it does
//  NOT matter to do EXTRAS, but probably WOULD matter to fail to do one when
//  it should be done. (TMValidSession calls should assign a bErrL of 20 or higher
//     if they fail.)

//===AAAAA==========================
//Consider calling TMValidSession... i.e. do it, unless bErrL shows we're
//   already past a place where an error makes doing more futile.
if bErrL=0 then begin// Yes... you can be more "clever"... why risk it?
siResult:=TMValidSession(liSessionHandle);// (instance 13c21a)
//... not really needed here, "should" still be
//    valid so soon after the call of TMExtenedSession. Here mainly to
//    illustrate how to use a TMValidSession should you want to insert one.

//Similar blocks can be "sprinkled" in what follows as you see fit,
//    but be careful with the bErrL and bErrL1 codes to distinguish
//    any failure from all others.

//TMEX API notes said TMValidSession returned 16-bit
     //UNsigned... but then gave example where "if (it)<0 then..."
     //appeared. So I am using Delphi smallint, which is 16 bit SIGNED

if siResult<0 then begin
        bErrL:=20;//Other error, see TMEX Session Error Return Codes
        //N.B.: For additional instances of this "call TMValidSession"
        //  code, different bErrL codes should be assigned.
          bErrL1:=240;
          if siResult=-200 then begin
             bErrL1:=200;
             sErrMsgL:='INVALID_SESSION -  session not valid';
             end;
          if siResult=-201 then begin
             bErrL1:=201;
             sErrMsgL:='HS_NOT_FOUND -  Required Hardware_Specific driver not found';
             end;
          if bErrL1=240 then begin
             bErrL1:=1;
             sErrMsgL:='Other error, not in Dallas docs, TMEX API ver 3.10';
             end;
        end;//siResult<0
        //end of flag problems code for call of TMValidSession
   end;//call TMValidSession (instance 13c21a)

//============= vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
// HERE BEGINS A BLOCK OF "maybe later" CODE. IF BROUGHT INTO USE, IT
// WILL NEED A LOT OF TESTING............
(*
//(Issue the Reset command)... an alternative way to reset the chips on the
//  MicroLan... but for SUBESQUENT resets, after a TMSetup has been done
//  since the previous TMExtendedSessionStart (without TMEndSession in between)

//Note that TMTouchReset must not be called before a TMSetup has been called...
//  but TMTouchReset can detect presence pulses, and distinguish between the
//  "alarming" and "not alarming" variations of them.

//This TMToughReset code not thoroughly tested.

siResult:=TMTouchReset(liSessionHandle);
//TMEX API notes said TMTouchReset returned 16-bit
     //UNsigned... but then gave example where "if (it)<0 then..."
     //appeared. So I am using Delphi smallint, which is 16 bit SIGNED

//After TMTouchReset, a slew of messages (via siResult) are possible.
if siResult=0 then begin
     //We are happy... the reset seems to have gone okay.
     //No presence pulse was detected... which doesn't concern us
     //     but I've put this bit of code in for future expansion
     end;
if siResult=1 then begin
     //We are happy... the reset seems to have gone okay.
     //A "non-alarming presence pulse" was detected... which doesn't concern us
     //     but I've put this bit of code in for future expansion.
     //     "Non-alarming presence pulse": Presence pulses come in two
     //         "flavors". There at least one chip to send a presence pulse,
     //         but no chip was sending an alarm as well.
     end;
if siResult=2 then begin
     //We are happy... the reset seems to have gone okay.
     //An "alarming presence pulse" was detected... which doesn't concern us
     //     but I've put this bit of code in for future expansion.
     //     "Alarming presence pulse": Presence pulses come in two
     //         "flavors". There was a chip which was sending a presence pulse,
     //         AND the chip was "alarming". (Generating the alarm message)
     end;

//Now we'll handle the various messages which can come back in siResult
//which will bother us... This is still following the call of TMTouchReset

if (bErrL=0)and (siResult=3) then begin
   bErrL:=40;

   end;//Deal with (bErrL=0)and (siResult=3)

if (bErrL=0)and (siResult=5) then begin
   bErrL:=41;

   end;//Deal with (bErrL=0)and (siResult=5)

if (bErrL=0)and (siResult<1) then begin
   bErrL:=240;
   if siResult=-1 then begin //-1
        bErrL:=44;
        bErrL1:=1;
        sErrMsgL:='Specified MicroLan has not been '+
           'initialized with a call to the TMSetup function';
        end;//-1

   if siResult=-2 then begin //-2
        bErrL:=44;
        bErrL1:=2;
        sErrMsgL:='Specified MicroLan nonexistent.';
        end;//-2

   if siResult=-3 then begin //-3
        bErrL:=44;
        bErrL1:=3;
        sErrMsgL:='Function not supported';
        end;//-3

   if siResult=-12 then begin //-12
        bErrL:=44;
        bErrL1:=12;
        sErrMsgL:='Failure to communicate with hardware adapter';
        end;//-12

   if siResult=-13 then begin //-13
        bErrL:=44;
        bErrL1:=13;
        sErrMsgL:='An unsolicited event occurred on the 1-Wire';
        end;//-13

   if siResult=-200 then begin //-200
        bErrL:=44;
        bErrL1:=200;
        sErrMsgL:='session not valid (not applicable in DOS TMEX)';
        end;//-200

   if siResult=-201 then begin //-201
        bErrL:=44;
        bErrL1:=201;
        sErrMsgL:='Hardware_Specific driver not found and is required';
        end;//-201

   if bErrL=240 then begin //(unrec -ve code)
        bErrL:=45;
        bErrL1:=99;
        sErrMsgL:='siResult was unrecognised (TMEX 3.10) error code '+
           inttostr(siResult);
        end;//(unrec -ve code)

      end;//Deal with (bErrL=0)and (siResult<1)
      //end of flag problems code for call of TMTouchReset

   end;//Issue "reset" to chips on MicroLan by TMTouchReset (Which can
       //  only be called after a TMSetup has been executed.

END OF UNTESTED "Issue TMTouchReset", the "maybe later" CODE.*)
//  ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
//--------------------------------------------------------------------



//===BBBBB==========================
//We're now "connected" to the adapter, the TMEX API... so
//   now we start sending commands to the chips on the
//   MicroLan...
//Consider resetting all 1-wire chips on the MicroLan
if bErrL=0 then begin
//Issue a "reset" to all of the chips on the MicroLan
//This is a bit like doing a power cycle to any bit
//    of mis-behaving electronics. Note: We are NOT
//    re-setting our "connection" to the TMEX API an
//    the adapter between the PC and the MicroLan...
//    We are just resetting all of the chips ON the
//    MicroLan.
//TMSetup also performs certain checks on the MicroLan,
//    such as "is it shorted"?

siResult:=TMSetup(liSessionHandle);
//TMEX API notes said TMTouchSetup returned 16-bit
     //UNsigned... but then gave example where "if (it)<0 then..."
     //appeared. So I am using Delphi smallint, which is 16 bit SIGNED

//After TMSetup, a slew of messages (via siResult) are possible.
if siResult=0 then begin
        bErrL:=40;
        bErrL1:=0;
        sErrMsgL:='TMSetup failed. That''s all that the '+
           'documentation says about the meaning of a returned zero. '+
           'Other return values are possible... with more information! '+
           'Zero seems to be a general "fail" when reason can''t be given.';
        end;
if siResult=1 then begin
     //We are happy... TMSetup went well.
     end;
if siResult=2 then begin
        bErrL:=41;
        bErrL1:=2;
        sErrMsgL:='TMSetup went ok. but MicroLan is shorted.';
     end;
if siResult=3 then begin
        bErrL:=44;
        bErrL1:=3;
        sErrMsgL:='TMSetup says MicroLan does not exist.';
        //This error message did NOT arise in some brief tests
        //  on a system WITH an iButtonLink LinkUSB plugged in...
        //  but with no MicroLan plugged into the LinkUSB
        //It DID occur if the LinkUSB was unplugged, and it was
        //  only necessary to re-connect the LinkUSB to the PC
        //  to have the program run without complaint again. (The
        //  program did not need to be re-started.)
     end;
if siResult=4 then begin
        bErrL:=45;
        bErrL1:=4;
        sErrMsgL:='TMSetup says TMSetup not supported by your hardware/OS.';
     end;

if (siResult<1) then begin
   bErrL:=240;
   if siResult=-1 then begin //-1
        bErrL:=46;
        bErrL1:=1;
        sErrMsgL:='Specified MicroLan has not been '+
           'initialized with a call to the TMSetup function';
        //It would be odd if this error arose during a call of TMSetup!
        //But it is a TMSetup/ TMTouchReset error message, and I am
        //  including a handler for it here, just in case.
        end;//-1

   if siResult=-2 then begin //-2
        bErrL:=46;
        bErrL1:=2;
        sErrMsgL:='Specified MicroLan nonexistent.';
        end;//-2

   if siResult=-3 then begin //-3
        bErrL:=46;
        bErrL1:=3;
        sErrMsgL:='Function not supported';
        end;//-3

   if siResult=-12 then begin //-12
        bErrL:=46;
        bErrL1:=12;
        sErrMsgL:='Failure to communicate with hardware adapter';
        end;//-12

   if siResult=-13 then begin //-13
        bErrL:=46;
        bErrL1:=13;
        sErrMsgL:='An unsolicited event occurred on the 1-Wire';
        end;//-13

   if siResult=-200 then begin //-200
        bErrL:=46;
        bErrL1:=200;
        sErrMsgL:='session not valid (not applicable in DOS TMEX)';
        end;//-200

   if siResult=-201 then begin //-201
        bErrL:=46;
        bErrL1:=201;
        sErrMsgL:='Hardware_Specific driver not found and is required';
        end;//-201

   if bErrL=240 then begin //(unrec -ve code)
        bErrL:=46;
        bErrL1:=99;
        sErrMsgL:='siResult was unrecognised (TMEX 3.10) error code '+
           inttostr(siResult);
        end;//(unrec -ve code)

      end;//Deal with (bErrL=0)and (siResult<1)
      //end of flag problems code for call of TMTouchReset

   end;//Issue "reset" to chips on MicroLan by TMSetup


//===BBBBB==========================
//Consider putting to sleep all of the chips APART FROM
//the one you want to. (This is how you can send commands
//down the wire they're all connected to without them all
//reacting.)

//Here's a little "wrinkle"...
//The TMEX command I am going to use, TMAccess, does a TMTouchReset
//  as part of itself (i.e. part of TMAccess)
//However, the (documented) return values which can arise from a
//  call of TMAccess do not include all of the values which can be
//  returned by a TMTouchReset.
//Rememember also that you can't do a TMTouchReset until you have
//  done a TMSetup... which ALSO resets all the chips on the MicroLan
//
//For now, I'm "going with" doing a TMSetup, as above, AND a TMAccess,
//  even though that means that the MicroLan gets an "extra" reset.
//This will entail a time cost. If you need hyper-fast reading of multiple
//  sensors, you may want to look into a different "skeleton" for your
//  program. If a TMSetup must happen after each TMExtendedStartSession,
//  you may find the headaches of doing one just once significant.

//(As before, the TMEX documentation is internally inconsistent. It says
//  that TMAccess returns a "short"-type value, in TMEX terms an UNsigned
//  16-bit number, but then goes on to say that it can return negative
//  numbers... so I used a Delphi smallint-type variable, capable of
//  handling SINGED 16-bit numbers.

//For the first time, one "step" of our plan is going to require TWO
//  TMEX calls. We will do a TMRom first, to put the unique ID of the chip
//  we want to stay awake where it is needed for the call of TMAccess which
//  comes after the call of TMRom.
//(The "unique ID" of the chip is the 16 hex character ID string which
//  we can use OneWireViewer to read from the chips we happen to have.)
//The chip id always ends with a two hex character "family ID" for the
//  sort of chip that it is, e.g. $10 for DS1920s, $28 for DS18B20s.
//  The family code for a given type of chip is given in the chip's datasheet.

//First, prepare the way, by putting the unique ID of the chip you DON'T
//  want asleep into the "StateBuffer" array.

//Start by preparing the way for a call of TMRom by the following. (bTmp is
//being used to select WHICH chip's ID will be passed to FillsiNumbersForTMRom)

//Before this call, there should be a sensible string in both sChipID[0] and
//   sChipID[1].. which SHOULD be the case, by this point, due to data vetting
//   in the eChipID OnChange handlers
bTmpL:=0;
if rbReadChip1.Checked then bTmpL:=1;
FillsiNumbersForTMRom(bTmpL);

if bErrL=0 then begin
  siResult:=TMRom(liSessionHandle,@bTMEXstatebuffer,@siNumbersForTMRom);
  //=== Anecdote... why programmers go gray....
  //31Dec13 into 1/1/14: I had "@FillsiNumbersForTMRom"... the address
  //     of the subroutine, rather than "@siNumbersForTMRom"... the address
  //     of the array. Took several hours to spot my mistake, mere
  //     seconds to remove. Sigh.

  //TMRom returns 1 if all well

  if siResult=-1 then begin
      bErrL:=50;
      bErrL1:=1;
      sErrMsgL:='PORT_NOT_INITIALIZED - Specified MicroLan has not '+
                 'been initialized with a call to the TMSetup function.';
      end;//siResult=-1
  if siResult=-2 then begin
      bErrL:=51;
      bErrL1:=2;
      sErrMsgL:='PORT_NOT_EXIST - Specified MicroLan nonexistent.';
      end;//siResult=-2
  if siResult=-3 then begin
      bErrL:=52;
      bErrL1:=3;
      sErrMsgL:='NO_SUCH_FUNCTION - Function not supported.';
      end;//siResult=-3
  if siResult=-200 then begin
      bErrL:=53;
      bErrL1:=200;
      sErrMsgL:='INVALID_SESSION - session not valid (not applicable in DOS TMEX)';
      end;//siResult=-200
  if siResult=-201 then begin
      bErrL:=54;
      bErrL1:=201;
      sErrMsgL:='HS_NOT_FOUND - Hardware_Specific driver not found and is required.';
      end;//siResult=-201
   //Now check that it wasn't an un-provided for error...
   if (bErrL=0) and (siResult<>1) then begin//TMRom returns 1 when all well
      bErrL:=56;
      bErrL1:=99;
      sErrMsgL:='TMRom returned an undocumented error flag: '+inttostr(siResult);
      end;//siResult=-201

      //end of taking care of error messages

   end;//End of first part of telling all but one of them to go to sleep


//Now that we've prepared the way, by putting the chip ID
//  of the chip we want left awake into the right part of
//  bTMEXstatebuffer, it is time for the TMAccess call...

if bErrL=0 then begin
  siResult:=TMStrongAccess(liSessionHandle,@bTMEXstatebuffer);
  //Returns 1 if all well: chip seen on MicroLan, and the chip awake
  //   while others sleep... "ROM" (the chip) "selected" in Dallas-speak

  //You have a choice here. You can use TMStrongAccess or TMAccess.
  //They both need the same parameters.
  //They both put all of the chips but one to sleep.
  //The error codes they return are ALMOST the same... the only
  //  difference being that with TMStrongAccess, if zero returned
  //  it means that the chip requested wasn't seen.
  //With TMAccess, if zero is returned it means that no chips were
  //  seen (at least nothing sent a "presence" pulse.
  //The downside of TMStrongAccess is that it takes three times as
  //  long to execute as TMAccess. For the extra security of knowing
  //  that the chip you expected to be there was found, I say the
  //  time is usually well spent!

  //Case of a zero returned taken care of at bottom

  if siResult=-1 then begin
      bErrL:=60;
      bErrL1:=1;
      sErrMsgL:='PORT_NOT_INITIALIZED - Specified MicroLan has not '+
                 'been initialized with a call to the TMSetup function.';
      end;//siResult=-1
  if siResult=-2 then begin
      bErrL:=61;
      bErrL1:=2;
      sErrMsgL:='PORT_NOT_EXIST - Specified MicroLan nonexistent.';
      end;//siResult=-2
  if siResult=-3 then begin
      bErrL:=62;
      bErrL1:=3;
      sErrMsgL:='NO_SUCH_FUNCTION - Function not supported.';
      end;//siResult=-3
  if siResult=-200 then begin
      bErrL:=63;
      bErrL1:=200;
      sErrMsgL:='INVALID_SESSION - session not valid (not applicable in DOS TMEX)';
      end;//siResult=-200
  if siResult=-201 then begin
      bErrL:=64;
      bErrL1:=201;
      sErrMsgL:='HS_NOT_FOUND - Hardware_Specific driver not found and is required.';
      end;//siResult=-201
  if siResult=0 then begin
      bErrL:=65;
      bErrL1:=0;
      sErrMsgL:='Chip not detected when TMStrongAccess tried to select';
      end;//siResult=0

  //Now check that it wasn't an un-provided-for error...
  if (bErrL=0) and
     (siResult<>1) and  //1 is what is returned when all well!
     (siResult<>0) then begin
      bErrL:=66;
      bErrL1:=99;
      sErrMsgL:='TMRom returned an undocumented error flag: '+inttostr(siResult);
      end;//siResult=-201

      //end taking care of error messages
  //showmessage(inttostr(siResult));

   end;//End of second part of telling all but one of them to go to sleep


//===CCCCC==================================================
//Consider doing things with the one chip that is awake
//N.B. This is the part of TalkTo1Wire where the "customizing"
//  occurs, to deal with different jobs, different chips.
//WHAT is done with the chip is determined by the value in wWhatAction

if bErrL=0 then begin//1
boWhatActionHandlerFound:=false;

//Do things with the chip which is not asleep...

//The "do it"s in this "Do things" block, selected by
//   wWhichAction, may be large, may
//   have multiple steps. Will be chip specific
//   (with some common themes.)

//If, say, reading from a sensor, postpone,
//   separate out into separate code,
//   USING the results from the sensor
//   until you are OUT of the "use MicroLan" stuff
//if (problem) then begin bErrL:=... 70-249 reserved for any reporting
                        //which may be needed in this block


//-------------------
if wWhatAction=$44 then begin //2 Issue "Convert T" and do short pause
boWhatActionHandlerFound:=true;

//KLUDGE... DS1820 code... TRIPLE CHECK, even if "works"...

//Send message to chip telling it to take a reading
siResult:=TMTouchByte(liSessionHandle,$44);

//if on parasitic, should go into strong pullup for 500ms
//  just after "Convert Tture" command issued.
//(Is there a way to know if it is on parasitic?)

KludgeDelay;
//if it was on strong pull up, should come out
//the timing of coming out of strong pull up
//  is not clear in MemoryFunctionsFlowChart...
//  shown as happening after "convert tture",
//  before the "do resets until master achieves reset

end;//2 if wWhatAction=$44 then...

//-------------------
if wWhatAction=$BE then begin //2 Read scratchpad and CS, check CS
   //There is no point in using this wWhatAction switch until after
   //  the chip concerned has been accessed previously with something
   //  like a $44 (Convert tture) command, to put something interesting
   //  into the scratchpad
boWhatActionHandlerFound:=true;

//KLUDGE... DS1820 code... TRIPLE CHECK, even if "works"...

//Now issue a "ReadScratchpad" cmnd..
//The "Master Tx reset?" in the DS1820 Memory Functions flowchart, just after
//   the "Master Rx data from scratchpad" is there to cover
//   the case of a programmer WANTING to abandon the reading of the
//   stream of 9 bytes (data + checksum byte) the DS1820 is generating.
//   There's no NEED to issue a reset, and it would only happen
//   if the programmer inserted a "do reset" into the design of the program.
siResult:=TMTouchByte(liSessionHandle,$BE);

//Read 8 bytes, throwing away last 6
siResult:=TMTouchByte(liSessionHandle,$FF);
bTtureLo:=siResult and $FF;
siResult:=TMTouchByte(liSessionHandle,$FF);
bTtureHi:=siResult and $FF;

siResult:=TMTouchByte(liSessionHandle,$FF);
siResult:=TMTouchByte(liSessionHandle,$FF);
siResult:=TMTouchByte(liSessionHandle,$FF);
siResult:=TMTouchByte(liSessionHandle,$FF);
siResult:=TMTouchByte(liSessionHandle,$FF);
siResult:=TMTouchByte(liSessionHandle,$FF);


//Get one byte checksum
siResult:=TMTouchByte(liSessionHandle,$FF);
//See if it was what it ought to be.

(*Same code fine for reading DS1820 and DS18B20...
  How the two bytes returned from the chip are interpreted has to be
  different for 1820 ($10) and 18B20 ($28)
  if (baDMLRomID[bIdx,0]=$10) then rsTmp:=rRawToC(bTmp,bTmp2);
  if (baDMLRomID[bIdx,0]=$28) then rsTmp:=rRawToC18B20(bTmp,bTmp2);
                              *)
   laRes3.caption:='The raw values from the chip were: '+
           inttostr(bTtureHi)+'  '+inttostr(bTtureLo);

   //Now convert values in bTtureLo, bTtureHi to HUNDRETHS of degrees Kelvin.
   //The Dallas chips work in Celcius, which is Kelvin +273.15 Using
   //centi-degrees K (inside the program) means that you
   //don't need the hassles of floating point numbers. (You can, of course,
   //display the ttures in any units, with a simple conversion occuring just
   //before the tture appears.)
   //Range: If a 16-bitunsigned variable is used for the tture, you can
   //express any tture from absolute zero to 655.35 Kelvin, which is about
   //382 Celcius, which will, I hope "do". 

   //While both the DS1820 (family code $10) and the DS18B20 (family code $28)
   //  are "driven" the same way, you have to interpret the results differently,
   //  hence the inclusion of the chip's family code in what follows...

   bTmpL:=0;
   if rbReadChip1.Checked then bTmpL:=1;
   wTtureCentiK:=wRawTTureToCentiK(bTtureLo,bTtureHi,
                    strtoint('$'+copy(sChipID[bTmpL],15,2)));
   laTtureResult.caption:='Tture is: '+sTtureCfrmCentiK(wTtureCentiK)+' degrees C';;

end;//2 if wWhatAction=$BE then...

//Here ends block of wWhatAction handlers. Be sure that one was
//  executed for the value in wWhatAction used to call
//  TalkTo1Wire this time....

if boWhatActionHandlerFound=false then begin //2
       bErrL:=240;
       //bErrL1:= not used here
       sErrMsgL:='Un-supported wWhatAction requested: $'+inttohex(wWhatAction,4);
       showmessage(sErrMsgL);
       end;//2

end;//Do things with the chip which is not asleep...

//===AAAAA==========================
//Consider calling TMEndSession, as we are done with MicroLan,
//so now release resources, release lock on it.

//NOTE THAT THIS ONE IS SPECIAL... sometimes we do it, even if
//  errors arose along the way. We only DON'T do it when we
//  didn't succeed in opening the session, which will be indicated
//  by a value in bErrL of between 1 and 19, inclusive.

if (bErrL=0) or (bErrL>19) then begin
siResult:=TMEndSession(liSessionHandle);
//TMEX API notes said TMEdSession returns 16-bit
       //UNsigned... but then gave example where "if (it)<0 then..."
       //appeared. So I am using Delphi smallint, which is 16 bit SIGNED
if siResult<0 then begin
      bErrL:=250;//For details of error see TMEX Session Error Return Codes
          bErrL1:=240;
          if siResult=-200 then begin
             bErrL1:=200;
             sErrMsgL:='INVALID_SESSION -  session not valid';
             end;
          if siResult=-201 then begin
             bErrL1:=201;
             sErrMsgL:='HS_NOT_FOUND -  Required Hardware_Specific driver not found';
             end;
          if bErrL1=240 then begin
             bErrL1:=1;
             sErrMsgL:='Other error, not in Dallas docs, TMEX API ver 3.10';
             end;
       end;//siResult<0
       //end of flag problems code for call of TMEndSession
   end;//call TMEndSession

//Done using the MicroLan, so now we do...
if bErrL<>0 then DoWhatReportingOfErrorYouWantTo;

/// ... and....
//NOW use things we may have collected while using the MicroLan,
//   e.g. sensor readings.... if bErrL is still zero at this point.
//   (If it isn't, we can't be sure that you did collect what you
//   went to the MicroLan for.
if bErrL=0 then begin
    DoThingsIfNeeded;
    //laRes3.caption:=inttostr(bTtureLo)+'  '+inttostr(bTtureHi);
    end;

end;//TalkTo1Wire;

procedure TDS048f1.eChipID0Change(Sender: TObject);
//Yes! I realise that this routine is not well written!
var bCount:byte;
    sTmpL:string;
begin
boGoodChipID0:=true;
sChipID[0]:=eChipID0.text;
if length(eChipID0.text)<>16 then
  begin
    boGoodChipID0:=false;
    end//no ; here
  else begin
    for bCount:=1 to 16 do begin
       sTmpL:=uppercase(eChipID0.text[bCount]);
       if pos(sTmpL,'0123456789ABCDEF')=0 then boGoodChipID0:=false;
       end;//for
    end;//else
SeeIfTimeToEnableReadChip;
end;//eChipIDChange

procedure TDS048f1.eChipID1Change(Sender: TObject);
//Yes! I realise that this routine is not well written!
var bCount:byte;
    sTmpL:string;
begin
boGoodChipID1:=true;
sChipID[1]:=eChipID1.text;
if length(eChipID1.text)<>16 then
  begin
    boGoodChipID1:=false;
    end//no ; here
  else begin
    for bCount:=1 to 16 do begin
       sTmpL:=uppercase(eChipID1.text[bCount]);
       if pos(sTmpL,'0123456789ABCDEF')=0 then boGoodChipID1:=false;
       end;//for
    end;//else
SeeIfTimeToEnableReadChip;
end;//eChipIDChange

procedure TDS048f1.KludgeDelay;
var lwTmp:longword;
begin
lwTmp:=gettickcount+650;
repeat
application.processmessages;
until gettickcount>lwTmp;
end;//KludgeDelay

function TDS048f1.wRawTTureToCentiK(
     bTtureLo,bTtureHi,bChipFam:byte):word;
var wTmpL:word;
    bHths:byte;
    boNegativeL:boolean;
    liTmpL:longint;//signed 32 bit
//Returns 65535	if there's a problem
//Otherwise returns the temperature indicted by the numbers
//  from the sensor... expressed in hundreTHs of degrees Kelvin.

//This routine looks arcane... and it may have errors in it!
//However, it is "no big deal".
//All we are doing here is taking the two numbers returned by
//  a Dallas 1-Wire tture sensor, and TRYING to apply the right
//  rules to convert them to a single number, closely related
//  to units used in the everyday world for measuring temperatures.

//In: two numbers, and a chip family ID
//Out: Temperature, in centiKelvins.
//Centi-Kelvin: hundreTH of a Kelvin degree.
//Zero Celcius is 273.15 Kelvin is 27315 centiKelvins
//100  Celcius is 373.15 Kelvin is 37312 centiKelvins

//THIS CODE HAS NOT BEEN TESTED AS THOROGHLY as the rest of this
//I (and others!) have had trouble with the code for
//this conversion in the past, with, in particular, problems
//converting values which should give negative ttures in C, and
//other special cases.

//All of the 1-Wire tture sensors work in Celsius. If you want
//Fahrenheit, you're best bet is to convert the raw bytes
//to C, and go to F from there*)

//Both the DS1820 and the DS18B20 will return 85degC if their
//tture register is read before the power-on reset value is changed.

begin
wTmpL:=65535;
if (bChipFam<>$10)and(bChipFam<>$28)then begin //1
  showmessage('wRawTTureToCentiK was sent a chip type it doesn''t know '+
  'what to do with. Will return a value of 65535');
  end; //end of deal with family code not $10 and not $28

if bChipFam=$10 then begin
//Convert bTtureLo and bTtureHi from a DS1820 to tture in centiKelvins

//This code for handling the two bytes from a DS1820,
//LSB of chip ID: $10. Examples taken from Dallas docs...
//I hope correctly... If code doesn't give these results,
//it is more likely the code that is wrong.

//In the table, the two bytes the DS1820 returns have been
//concatenated to a single 16-bit number.

//$0001 is 0.5 degree C  (27365 centidegrees K)
//$0007 is 3.5 d C       (27665 centidegrees K)
//$FFFF is -0.5 d C      (27265 centidegrees K)

//This algorithm only interprets the tture to 0.5 degree C.
//See data sheet for how to get greater resolution.

  //bTtureHi:=$0;//for debug
  //bTtureLo:=$0;//for debug

//Start by finding tture in hundreths of degrees Celcius

  if (bTtureLo and 1)=1 then
    bHths:=50 else bHths:=0;
  if bTtureHi=$FF then begin (*negative tture*)
    bTtureLo:=(NOT bTtureLo)+1;
    liTmpL:=(((bTtureLo div 2)*100)+bHths)*-1;
    end //no ; here. Done doing negative number
  else begin //+ve tture
    liTmpL:=((bTtureLo div 2)*100)+bHths;
  end;(*else*)

  //At this point, liTmpL holds tture in centidegrees C
  //showmessage(inttostr(liTmpL));

//Next part also used in code for DS18B20... should move to a function
//  either can call

  //Be sure we aren't headed for a tture of below
  //   absolute zero, which is 0 degrees K
  if liTmpL<-27315 then begin
    //do nothing if <-27315.. flow will fall through, with
    //wTmpL still set to rogue for "problem"
    end//no ; here
   else begin//liTmpL was >= -27315... good!
  //Convert to centidegrees C to centidegrees K... to avoid the need
  //for negative numbers... dealing in centidegrees spared us fractions
    liTmpL:=liTmpL+27315;
    end;//.. of the else for liTmpL was >= -27315

  //liTmpL:=1000;//1000 centiK= 10 deg K:= -263.15 deg C<<for debug

  wTmpL:=liTmpL;
end;//bChipFam=$10

if bChipFam=$28 then begin
//Convert bTtureLo and bTtureHi from a DS18B20 to tture in centiKelvins

//This code for handling the two bytes from a DS18B20,
//LSB of chip ID: $28. Examples taken from Dallas docs...
//I hope correctly... If code doesn't give these results,
//it is more likely the code that is wrong.

//In the table, the two bytes the DS18B20 returns have been
//concatenated to a single 16-bit number.

//$07D0 is 125.00 degrees C
//$0008 is   0.5  degree C  (27365 centidegrees K)
//$0000 is   0.000 d C      (27315 centidegrees K)
//$FFF8 is - 0.500 d C      (27265 centidegrees K)
//$FF5E is -10.125 d C
//$FC90 is -55.000 dC

//if (bTtureHi and $F8) = $F8, the number is a negative number,
//  given in two's complement form.

//The LS 4 bits give the fractional part of the tture.

//The theoretical resolution of the hardware is 1/1th degree, but
//  this algorithym doesn't provide for thousandths of degrees

  //bTtureHi:=$0;//for debug
  //bTtureLo:=$0;//for debug



//Start by finding tture in hundreths of degrees Celcius

//Combine MS and LS bytes in wTmpL, remembering that at
//  the end of the SR that must be 65535 if conversion has failed, something
//  else if conversion has not failed.
//Decide if the number is +ve or negative, and use if.. then.. to temporarily
//  divide flow accordingly at this point after setting boNegativeL

//For +ve number... nothing to do at this stage
//For -ve number... do two's complement

//Calculate the centidegrees Celcius, forgetting whether number had 2's complement
//Move into liTmpL (signed) and multiply by -1 if boNegativeL is true.

//Now that tture in centidegrees C is in liTmpL, convert to centidegrees K,
//  and return that to wTmpL... or, as I said at start, make it 65535
//  if conversion has failed.

wTmpL:=bTtureHi*$FF+bTtureLo;

boNegativeL:=false;
if (wTmpL and $F800)=$F800 then begin
  boNegativeL:=true;
  wTmpL:=(NOT wTmpL)+1;
  end;

//Yes... this could be done better!
bTtureLo:=wTmpL AND $0F;//re-purposed to hold fraction part.
if bTtureLo=$0 then bHths:=0;//Exactly 0
if bTtureLo=$1 then bHths:=6;//
if bTtureLo=$2 then bHths:=12;//0.125
if bTtureLo=$3 then bHths:=18;//
if bTtureLo=$4 then bHths:=25;//Exactly 0.25
if bTtureLo=$5 then bHths:=32;//
if bTtureLo=$6 then bHths:=38;//0.375
if bTtureLo=$7 then bHths:=44;//
if bTtureLo=$8 then bHths:=50;//Exactly 0.5
if bTtureLo=$9 then bHths:=56;//
if bTtureLo=$10 then bHths:=62;//0.625
if bTtureLo=$11 then bHths:=68;//
if bTtureLo=$12 then bHths:=75;//Exactly 0.75
if bTtureLo=$13 then bHths:=82;//
if bTtureLo=$14 then bHths:=88;//0.875
if bTtureLo=$15 then bHths:=94;//

liTmpL:=wTmpL;//32 bit signed
liTmpL:=(liTmpL div 16)*100;//Integer part now in centidegrees C
liTmpL:=liTmpL+bHths;//add in the previously determined 100ths.
if boNegativeL then liTmpL:=litmpL*-1;//Fully realized tture in C centidegrees
//showmessage(inttostr(liTmpL));<<for prgm testing

//Next part also used in code for DS1820... should move to a function
//  either can call

  //Be sure we aren't headed for a tture of below
  //   absolute zero, which is 0 degrees K
  if liTmpL<-27315 then begin
    //do nothing if <-27315.. flow will fall through, with
    //wTmpL still set to rogue for "problem"
    end//no ; here
   else begin//liTmpL was >= -27315... good!
  //Convert to centidegrees C to centidegrees K... to avoid the need
  //for negative numbers... dealing in centidegrees spared us fractions
    liTmpL:=liTmpL+27315;
    end;//.. of the else for liTmpL was >= -27315

  //liTmpL:=1000;//1000 centiK= 10 deg K:= -263.15 deg C<<for debug

  wTmpL:=liTmpL;


//  wTmpL:=1100;//1100 centiK= 11 deg K:= -262.15 deg C
end;//bChipFam=$10

result:=wTmpL;//Tture in centiKelvins... or 65535 on error
end; //wRawTTureToCentiK

function TDS048f1.sTtureCfrmCentiK(wTture:word):string;
//Supplied with tture in centidegrees Kelvin,
//Returns a string with that tture in Celcius

//This could be written better! But it works!

var sTmpL,sTmpL1:string;

begin                     
if wTture<27315 then begin //Process something that will be a negative
  wTture:=27315-wTture;
  sTmpL:=inttostr(wTture);
  if length(sTmpL)<2 then sTmpL:='0'+sTmpL;
  sTmpL1:=copy(sTmpL,1,length(sTmpL)-2)+'.'+copy(sTmpL,length(sTmpL)-1,2);
  if sTmpL1[1]='.' then sTmpL1:='0'+sTmpL1;
  sTmpL1:='-'+sTmpL1;
  end//no ; here. Negative number dealt with
 else begin
  wTture:=wTture-27315;
  sTmpL:=inttostr(wTture);
  if length(sTmpL)<2 then sTmpL:='0'+sTmpL;
  sTmpL1:=copy(sTmpL,1,length(sTmpL)-2)+'.'+copy(sTmpL,length(sTmpL)-1,2);
  if sTmpL1[1]='.' then sTmpL1:='0'+sTmpL1;
  end;//deal with tture at or above 0 deg C
result:=sTmpL1;
end;//sTtureCfrmCentiK

end.
